@@ -130,6 +130,13 @@ HONGGFUZZ_ASAN_JOB = JobInfo(
     'address', ['honggfuzz', 'engine_asan'],
     minimize_job_override=LIBFUZZER_ASAN_JOB)
 
+GFT_ASAN_JOB = JobInfo('googlefuzztest_asan_', 'googlefuzztest', 'address',
+                       ['googlefuzztest', 'engine_asan'])
+GFT_MSAN_JOB = JobInfo('googlefuzztest_msan_', 'googlefuzztest', 'memory',
+                       ['googlefuzztest', 'engine_msan'])
+GFT_UBSAN_JOB = JobInfo('googlefuzztest_ubsan_', 'googlefuzztest', 'undefined',
+                        ['googlefuzztest', 'engine_ubsan'])
+
 JOB_MAP = {
     'libfuzzer': {
         'x86_64': {
@@ -151,6 +158,13 @@ JOB_MAP = {
             'address': HONGGFUZZ_ASAN_JOB,
         },
     },
+    'googlefuzztest': {
+        'x86_64': {
+            'address': GFT_ASAN_JOB,
+            'memory': GFT_MSAN_JOB,
+            'undefined': GFT_UBSAN_JOB,
+        },
+    },
     'none': {
         'x86_64': {
             'address': NO_ENGINE_ASAN_JOB,
@@ -563,6 +577,7 @@ class ProjectSetup(object):
                config_suffix='',
                external_config=None,
                segregate_projects=False,
+               experimental_sanitizers=None,
                engine_build_buckets=None,
                fuzzer_entities=None,
                add_info_labels=False,
@@ -574,6 +589,7 @@ class ProjectSetup(object):
     self._build_bucket_path_template = build_bucket_path_template
     self._revision_url_template = revision_url_template
     self._segregate_projects = segregate_projects
+    self._experimental_sanitizers = experimental_sanitizers
     self._engine_build_buckets = engine_build_buckets
     self._fuzzer_entities = fuzzer_entities
     self._add_info_labels = add_info_labels
@@ -713,6 +729,9 @@ class ProjectSetup(object):
             'reproduction_topic']
         job.external_updates_subscription = self._external_config[
             'updates_subscription']
+      else:
+        job.external_reproduction_topic = None
+        job.external_updates_subscription = None
 
       if not info.get('disabled', False):
         job_names.append(job_name)
@@ -776,7 +795,9 @@ class ProjectSetup(object):
       if help_url:
         job.environment_string += 'HELP_URL = %s\n' % help_url
 
-      if template.experimental:
+      if (template.experimental or
+          (self._experimental_sanitizers and
+           template.memory_tool in self._experimental_sanitizers)):
         job.environment_string += 'EXPERIMENTAL = True\n'
 
       if template.minimize_job_override:
@@ -944,12 +965,25 @@ class Handler(base_handler.Handler):
       logs.log_error('Failed to get honggfuzz Fuzzer entity.')
       return
 
+    gft = data_types.Fuzzer.query(
+        data_types.Fuzzer.name == 'googlefuzztest').get()
+    if not gft:
+      logs.log_error('Failed to get googlefuzztest Fuzzer entity.')
+      return
+
     project_config = local_config.ProjectConfig()
     segregate_projects = project_config.get('segregate_projects')
     project_setup_configs = project_config.get('project_setup')
     project_names = set()
     job_names = set()
 
+    fuzzer_entities = {
+        'afl': afl,
+        'honggfuzz': honggfuzz,
+        'googlefuzztest': gft,
+        'libfuzzer': libfuzzer,
+    }
+
     for setup_config in project_setup_configs:
       bucket_config = setup_config.get('build_buckets')
 
@@ -963,19 +997,18 @@ class Handler(base_handler.Handler):
           config_suffix=setup_config.get('job_suffix', ''),
           external_config=setup_config.get('external_config', ''),
           segregate_projects=segregate_projects,
+          experimental_sanitizers=setup_config.get('experimental_sanitizers',
+                                                   []),
           engine_build_buckets={
               'libfuzzer': bucket_config.get('libfuzzer'),
               'libfuzzer-i386': bucket_config.get('libfuzzer_i386'),
               'afl': bucket_config.get('afl'),
               'honggfuzz': bucket_config.get('honggfuzz'),
+              'googlefuzztest': bucket_config.get('googlefuzztest'),
               'none': bucket_config.get('no_engine'),
               'dataflow': bucket_config.get('dataflow'),
           },
-          fuzzer_entities={
-              'libfuzzer': libfuzzer,
-              'honggfuzz': honggfuzz,
-              'afl': afl,
-          },
+          fuzzer_entities=fuzzer_entities,
           add_info_labels=setup_config.get('add_info_labels', False),
           add_revision_mappings=setup_config.get('add_revision_mappings',
                                                  False),
@@ -996,5 +1029,6 @@ class Handler(base_handler.Handler):
       project_names.update(result.project_names)
       job_names.update(result.job_names)
 
-    cleanup_stale_projects([libfuzzer, afl, honggfuzz], project_names,
-                           job_names, segregate_projects)
+    cleanup_stale_projects(
+        list(fuzzer_entities.values()), project_names, job_names,
+        segregate_projects)
